/**
  ******************************************************************************
  * @file    SerCom.c
  * @author  ingo_s
  * @version V1.0.0
  * @date    2015.01.15
  * @brief   function for serial I/O of STRootLoader
  ******************************************************************************
   *
  * This file is free software: you can redistribute it and/or modify
  * it under the terms of the version 3 GNU General Public License as
  * published by the Free Software Foundation.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY.
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  ******************************************************************************
*/

/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx.h"
#include "stm32f0xx_gpio.h"
#include "stm32f0xx_usart.h"
#include "stm32f0xx_rcc.h"

#include "sercom.h"
#include "config.h"

#ifndef USART_HardwareFlowControl_HDSEL
#define USART_HardwareFlowControl_HDSEL		 ((uint16_t)0x0008)
#endif

/* export functions ---------------------------------------------------------*/

/**
  * @brief Init used UART and PORT
  * @param  None
  * @retval None
  */
// Test - noch 2wire Mode
void WIRE_Init(void)
{
	  GPIO_InitTypeDef GPIO_InitStructure;
	  USART_InitTypeDef USART_InitStructure;

	  /* Enable GPIO clock */
	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);
#if defined(USART1_BL)
	  /* Enable USART clock */
	  RCC_APB2PeriphClockCmd( RCC_APB2Periph_USART1, ENABLE);
#else
	  /* Enable USART clock */
	  RCC_APB1PeriphClockCmd( RCC_APB1Periph_USART2, ENABLE);
#endif
	  /* Connect PXx to USARTx_Tx */
	  GPIO_PinAFConfig(GPIO_BL, GPIO_PinSourceTX, GPIO_AF_1);
#ifndef USART_ONE_WIRE
	  /* Connect PXx to USARTx_Rx */
	  GPIO_PinAFConfig(GPIO_BL, GPIO_PinSourceRX, GPIO_AF_1);
#endif
	  /* Configure USART Tx as alternate function push-pull */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_TX;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIO_BL, &GPIO_InitStructure);
#ifndef USART_ONE_WIRE
	  /* Configure USART Rx as alternate function input */
	  GPIO_StructInit(&GPIO_InitStructure);
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_RX;
	  GPIO_Init(GPIO_BL, &GPIO_InitStructure);
#endif
	  /* USART configuration */
	  USART_InitStructure.USART_BaudRate = BAUDRATE;
	  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	  USART_InitStructure.USART_StopBits = USART_StopBits_1;
	  USART_InitStructure.USART_Parity = USART_Parity_No;
#ifdef USART_ONE_WIRE
	  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_HDSEL;
	  USART_InitStructure.USART_Mode = USART_Mode_Rx /*| USART_Mode_Tx*/;
#else
	  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
#endif
	  USART_Init(USART_BL, &USART_InitStructure);

	  /* Enable USART */
	  USART_Cmd(USART_BL, ENABLE);

}

/**
  * @brief  deInit used UART and PORT to reset status
  * @param  None
  * @retval None
  */
void WIRE_DeInit(void)
{
	// set UART to reset status
	USART_Cmd(USART_BL, DISABLE);
	USART_DeInit(USART_BL);
	USART_BL->CR1 = 0;
#if defined(USART1_BL)
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_USART1, DISABLE);
#else
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_USART2, DISABLE);
#endif
	// set GPIOn to reset status
	GPIO_DeInit(GPIO_BL);
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, DISABLE);
}

/**
  * @brief  get char from serial input
  * @param  *destination for char
  * @retval 1: char received
  * 		0: no char received
  */
uint8_t WIRE_getc(uint8_t *c)
{
#ifdef USART_ONE_WIRE
	// if TX activ wait for transmission complete, disable Tx, enable Rx
	if (USART_BL->CR1 & USART_Mode_Tx){
		while (!(USART_BL->ISR & USART_FLAG_TC))	{};
		USART_BL->CR1 &= ~USART_Mode_Tx;
		USART_BL->CR1 |= USART_Mode_Rx;
	}
#endif
	if ( USART_GetFlagStatus(USART_BL, USART_FLAG_RXNE) != RESET){
		*c = USART_ReceiveData(USART_BL);
		return 1;
	}
	else
		return 0;
}

/**
  * @brief  put char to serial output
  * @param  char
  * @retval None
  */
void WIRE_putc(uint8_t c)
{
#ifdef USART_ONE_WIRE
	// if Tx not activ, disable Rx, enable Tx
	if (!(USART_BL->CR1 & USART_Mode_Tx)){
		USART_BL->CR1 &= ~USART_Mode_Rx;
		USART_BL->CR1 |= USART_Mode_Tx;
	}
#endif
	USART_SendData(USART_BL, c);
	while (USART_GetFlagStatus(USART_BL, USART_FLAG_TXE) == RESET)	{};
}

/**
  * @brief  put string to serial output
  * @param  *char buffer
  * @param  number of chars
  * @retval None
  */
void WIRE_putstr(uint8_t* pChar, uint16_t len)
{
	for (int i = len; i>0; i--){
		WIRE_putc(*pChar);
		pChar++;
	}
}

/* ------------------------------------ EOF --------------------------------- */
