/**
  ******************************************************************************
  * @file    main.c
  * @author  ingo_s
  * @version V1.0.0
  * @date    2015.01.20
  * @brief   main file of STRootLoader
  * The code based of the ST AN4065 "STM32F0xx in-application programming (IAP)"
  ******************************************************************************
  *
  * This file is free software: you can redistribute it and/or modify
  * it under the terms of the version 3 GNU General Public License as
  * published by the Free Software Foundation.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY.
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  ******************************************************************************
*/

#include "stm32f0xx.h"
#include "stm32f0xx_gpio.h"
#include "stm32f0xx_rcc.h"

#include "config.h"
#include "sercom.h"

// ============================= Flash Section =================================
/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx_flash.h"

/* Private constants --------------------------------------------------------*/
#define FLASH_START 	0x08000000
#define FLASH_PAGE_SIZE  (uint16_t)0x400  /* Page size = 1KByte */
#define FLASH_SIZE_DR 0x1FFFF7CC

/* Private function prototypes -----------------------------------------------*/
static void Flash_getInfo(void);
static void Flash_Init(void);
static uint32_t Flash_Write(__IO uint32_t* FlashAddress, uint32_t* Data, uint16_t DataLength);

/* Private variables ---------------------------------------------------------*/
static uint32_t flash_size;
static uint32_t flash_end_adress;

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  get Flash size and Flash end address
  * @param  None
  * @retval None
  */
void Flash_getInfo(void)
{
	  /* get Flash size */
	  flash_size = (*(__IO uint16_t*)FLASH_SIZE_DR) * FLASH_PAGE_SIZE;
	  flash_end_adress = flash_size + FLASH_START;
}

/**
  * @brief  Unlocks Flash for write access
  * @param  None
  * @retval None
  */
void Flash_Init(void)
{
  /* Unlock the Program memory */
  FLASH_Unlock();
  /* Clear all Flash flags */
  FLASH_ClearFlag(FLASH_FLAG_EOP|FLASH_FLAG_WRPERR | FLASH_FLAG_PGERR | FLASH_FLAG_BSY);
}

/**
  * @brief  This function writes a data buffer in flash (data are 32-bit aligned).
  * @note   After writing data buffer, the flash content is checked.
  * @param  FlashAddress: start address for writing data buffer
  * @param  Data: pointer on data buffer
  * @param  DataLength: length of data buffer (unit is 32-bit word)
  * @retval 0: Data successfully written to Flash memory
  *         1: Error occurred while writing data in Flash memory
  *         2: Written Data in flash memory is different from expected one
  *         3: error on erase page
  */
uint32_t Flash_Write(__IO uint32_t* FlashAddress, uint32_t* Data ,uint16_t DataLength)
{
  uint32_t i = 0;

  for (i = 0; (i < DataLength) && (*FlashAddress <= (flash_end_adress-4)); i++){
	/* if page start, erase page */
	if ((*FlashAddress & (FLASH_PAGE_SIZE-1)) == 0){
		if (FLASH_ErasePage(*FlashAddress) != FLASH_COMPLETE){
			return (3);
		}
	}
    /* the operation will be done by word */
    if (FLASH_ProgramWord(*FlashAddress, *(uint32_t*)(Data+i)) == FLASH_COMPLETE){
     /* Check the written value */
      if (*(uint32_t*)*FlashAddress != *(uint32_t*)(Data+i)){
        /* Flash content doesn't match SRAM content */
        return(2);
      }
      /* Increment FLASH destination address */
      *FlashAddress += 4;
    }
    else{
      /* Error occurred while writing data in Flash memory */
      return (1);
    }
  }
  return (0);
}

/**
  * @brief  This function verify a data buffer with flash data (data are 32-bit aligned).
  * @param  FlashAddress: start address for writing data buffer
  * @param  Data: pointer on data buffer
  * @param  DataLength: length of data buffer (unit is 32-bit word)
  * @retval 0: Data verify successfully
  *         1: verify Error occurred
  */
uint32_t Flash_Verify(__IO uint32_t* FlashAddress, uint32_t* Data ,uint16_t DataLength)
{
  uint32_t i = 0;

  for (i = 0; (i < DataLength) && (*FlashAddress <= (flash_end_adress-4)); i++){
     /* Check the written value */
      if (*(uint32_t*)*FlashAddress != *(uint32_t*)(Data+i)){
        /* Flash content doesn't match SRAM content */
        return(1);
      }
      /* Increment FLASH destination address */
      *FlashAddress += 4;
  }
  return (0);
}


// ====================== end of Flash Section =================================
// -----------------------------------------------------------------------------
volatile uint32_t time_1ms=0;	// 1ms counter

void SysTick_Handler()
{
	time_1ms++;
}
// =============================================================================

/* Private constants ---------------------------------------------------------*/
#ifdef BOOTINFO_STRING
const char BootldrInfo[] = BOOTINFO_STRING;
#endif
const char BootSign[] = BOOTSIGN;
// AVR fake uP info string for 16k/32k/64k STM32F05x
const uint8_t BootInfo16k[4] = {0x94,0x03,0x06,0x04}; // Atmega16
const uint8_t BootInfo32k[4] = {0x95,0x02,0x06,0x04}; // Atmega32
const uint8_t BootInfo64k[4] = {0x96,0x02,0x06,0x04}; // Atmega64
//const uint8_t BootInfo64k[4] = {0x99,0x02,0x06,0x04}; // test STM32F051K8

#define SUCCESS			 0x30
#define	ERRORCOMMAND	 0xC1
#define	ERRORCRC		 0xC2
#define	ERRORPROG		 0xC5

/* Private variables ---------------------------------------------------------*/
static uint16_t crc = 0;
static uint32_t destadr = 0;
static uint16_t fill_len = 0;

static uint8_t cmd_in[10];

#ifdef STM32F0XX_LD
static uint8_t dbuf[2048];
#else
static uint8_t dbuf[4096];
#endif

static uint8_t* pBuf;

uint32_t JumpAddress;

/* Private function prototypes -----------------------------------------------*/
typedef  void (*pFunction)(void);
pFunction Jump_To_Application;

static uint16_t crc16_update(uint16_t crc, uint8_t a);
static void crc_getc(uint8_t* buf);
static uint8_t* crc_getdata(uint8_t* buf, uint16_t len);
static uint8_t eepromREAD(void);

static uint8_t connect(void);
static void StartApp(void);

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  crc16 update function
  * @note   Optimized CRC-16 calculation.
  * @note   Polynomial: x^16 + x^15 + x^2 + 1 (0xa001)
  * @param  last crc
  * @param  last byte value
  * @retval new crc
  */
uint16_t crc16_update(uint16_t crc, uint8_t a)
{
	int i;

	crc ^= a;
	for (i = 0; i < 8; ++i){
		if (crc & 1)
			crc = (crc >> 1) ^ 0xA001;
		else
			crc = (crc >> 1);
	}
	return crc;
}
// ----------------------------------------------------------------------------

/**
  * @brief  get one seriell input value
  * @param  *destination
  * @retval None
  */
void crc_getc(uint8_t* buf)
{
	uint8_t c;
	while (WIRE_getc(&c) == 0);
	crc = crc16_update(crc, c);
	*buf = c;
}

/**
  * @brief  get counted seriell input values
  * @param  *destination
  * @param  len / nuber of values
  * @retval next *destination
  */
uint8_t* crc_getdata(uint8_t* buf, uint16_t len)
{
	uint8_t* pbuf = buf;
	for (uint16_t i=0; i<len; i++)
	{
		crc_getc(pbuf);
		pbuf++;
	}
	return pbuf;
}

// ----------------------------------------------------------------------------
#ifdef EEPROM_READ_WRITE
/**
  * @brief  read one page of my EEPROM simulation page
  * @note	Host first SetAddress, then call readEeprom for no of pages (see Ini-file)
  * @note	my Eeprom simulation data are on flash size - EEPROM_SIZE
  * @retval 0: no error; 1: address to high
  */
uint8_t eepromREAD(void)
{
	if (destadr < FLASH_START)
		destadr |= (FLASH_START + flash_size - EEPROM_SIZE);
	if (destadr >= flash_end_adress)
		return (1);
	pBuf = dbuf;
	// copy one EEPROM page to Buffer (256 byte)
	for (int i=0; i<256; i++){
		*pBuf = *(uint8_t*)destadr;
		pBuf++;
		destadr++;
	}
	// make crc
	crc = 0;
	for (int i=0; i<256; i++)
		crc = crc16_update(crc, dbuf[i]);
	dbuf[256] = crc & 0xFF;
	dbuf[257] = (crc>>8);
	// send one Page
	WIRE_putstr(dbuf, 258);
	return (0);
}
#endif
// ----------------------------------------------------------------------------
/**
  * @brief  try to connect with timeout
  * @param  none
  * @retval 1: connected
  * 		0: not connected
  */
uint8_t connect(void)
{
	uint8_t c;
	uint8_t i_bs = 0;
	crc = 0;

	uint32_t waittime = time_1ms + CONNECT_TIMEOUT_MS;
	do
	{
		if(WIRE_getc(&c)){
			crc = crc16_update(crc, c);
			if (i_bs < sizeof(BootSign)-1){
				/* compare next char with BootSign */
				if (BootSign[i_bs] == c){
					i_bs++;
				}
				else{
					/* not equal, set start condition */
					i_bs= 0;
					crc = 0;
				}
			}
			else{
				/* on end of BootSign, compute crc only */
				if (i_bs == sizeof(BootSign)-1){
					i_bs++;
				}
				else{
					if (crc == 0)
						return 1;		// crc == 0, connected!
					else
						return 0;
				}
			}
		}
	}while (time_1ms < waittime);

	return 0;
}

// ----------------------------------------------------------------------------

/**
  * @brief  start application firmware
  * @param  None
  * @retval None
  */
void StartApp(void)
{
    /* Test if user code is programmed starting from address "APPLICATION_ADDRESS" */
    if (((*(__IO uint32_t*)APPLICATION_ADDRESS) & 0x2FFE0000 ) == 0x20000000)
    {
		/* restore I/O settings */
		FLASH_Lock();
		SysTick->CTRL = 0;
		WIRE_DeInit();

		/* Jump to user application */
		JumpAddress = *(__IO uint32_t*) (APPLICATION_ADDRESS + 4);
		Jump_To_Application = (pFunction) JumpAddress;

		/* Initialize user application's Stack Pointer */
		__set_MSP(*(__IO uint32_t*) APPLICATION_ADDRESS);

		/* Jump to application */
		Jump_To_Application();
    }
    /* Infinite loop */
    while (1)
    {
    }
}

/* ************************************************************************** */

int main(void)
{
	uint8_t state = 0;

	if (!(RCC->CSR & 0x0C000000))			// if not PwrOn or NRST, start App direct
		StartApp();
	// Info: App should reset the reset flags
	// RCC->CSR |= 0x01000000;

	// set default BootInfo 64k Flash
	uint8_t* pBootInfo = (uint8_t*)&BootInfo64k;

	/* get info about flash size */
	Flash_getInfo();
	/* adjust boot-info* if flash size < 64k */
	if (flash_size <= 0x7FFF)
		pBootInfo = (uint8_t*)&BootInfo16k;
	else if (flash_size <= 0xFFFF)
		pBootInfo = (uint8_t*)&BootInfo32k;

	/* init serial I/O */
	WIRE_Init();

	/* start Systic timer */
	SysTick_Config(SystemCoreClock / 1000);

	while(1)
    {
		if (!connect()){	// try to connect with timeout
			break;
		}
#ifdef BOOTINFO_STRING
		WIRE_putstr((uint8_t*)&BootldrInfo, sizeof(BootldrInfo));
#endif
		// send boot-info
		WIRE_putstr((uint8_t*)pBootInfo, sizeof(BootInfo64k));
		WIRE_putc(SUCCESS);

		Flash_Init();					// open flash
		do{
			/* get next AVRootLoader command */
			crc = 0;
			pBuf = cmd_in;
			crc_getc(pBuf++);
			crc_getc(pBuf++);
			crc_getc(pBuf++);
			crc_getc(pBuf++);
			// command == first byte of command string
			state = cmd_in[0];
			if (state > 0xFD){			// 6 byte command?
				crc_getc(pBuf++);		// get next 2 bytes
				crc_getc(pBuf++);
			}
			if (crc != 0x00){			// crc check of command
				WIRE_putc(ERRORCRC);
				continue;
			}

			// ## host command info: ---------------------------------------
			// 00 start firmware:		0x00, 0x01, crc, crc
			// 01 writeBuf2Flash:		0x01, 0x01, crc, crc
			// 02 erase flash:			0x02, 0x??, crc, crc (not supported)
			//    if used, setAddress are always 0!, 2nd byte look like no of pages
			// 03 verify flash:			0x03, 0x01, crc, crc
			// 04 EepromRead:			0x04, 0x00, crc, crc (simulation)
			// 05 EepromWrite:			0x05, 0x01, crc, crc (simulation)
			// 06 ReadSRam:				0x06, 0x00, crc, crc (not supported)
			// FD keep alive:			0xFD, 0x00, crc, crc
			// FE setBuffer len+data:	0xFE, 0x00, lenh, lenl, crc, crc,
			//			  followed by: 	data, ...., data,  crc, crc
			// FF setAddress			0xFF, adr2, adr1, adr0, crc, crc
			// -------------------------------------------------------------

			switch (state)				// state == command#
			{
			case 0x00:		// start firmware
					if (cmd_in[1] != 0x01)
						WIRE_putc(ERRORCOMMAND);
					else{
						// start application firmware
						StartApp();
					}
				break;
			case 0x01:		// write buf to flash
					destadr |= FLASH_START;
					/* some times AVRootLoadeer send 0 address */
					//if (destadr < APPLICATION_ADDRESS)
					//	destadr = APPLICATION_ADDRESS;
					if (cmd_in[1] != 0x01 || destadr < APPLICATION_ADDRESS)
						WIRE_putc(ERRORCOMMAND);
					else{
						if (Flash_Write((uint32_t*)&destadr, (uint32_t*)&dbuf, fill_len/4)){
							WIRE_putc(ERRORPROG);
						}else
							WIRE_putc(SUCCESS);
					}
				break;
			case 0x02:		// erase flash command
					// 2nd byte info unknown
					WIRE_putc(ERRORCOMMAND);
				break;
			case 0x3:		// verify flash
					if (fill_len){
						destadr |= FLASH_START;
						if (Flash_Verify((uint32_t*)&destadr, (uint32_t*)&dbuf, fill_len/4)){
							WIRE_putc(ERRORPROG);
						}else
							WIRE_putc(SUCCESS);
					}else
						WIRE_putc(SUCCESS);
					fill_len = 0;
				break;
#ifdef EEPROM_READ_WRITE
			case 0x04:		// eeprom read
					if (cmd_in[1] != 0x00)
						WIRE_putc(ERRORCOMMAND);
					else{
						if (eepromREAD() == 0)
							WIRE_putc(SUCCESS);
						else
							WIRE_putc(ERRORCOMMAND);
					}
					break;
			case 0x05:		// write buf to eeprom
					if (cmd_in[1] != 0x01)
						WIRE_putc(ERRORCOMMAND);
					else{
						if (destadr < FLASH_START)
							destadr += (flash_size + (FLASH_START - EEPROM_SIZE));
						if (Flash_Write((uint32_t*)&destadr, (uint32_t*)&dbuf, fill_len/4)){
							WIRE_putc(ERRORPROG);
						}else
							WIRE_putc(SUCCESS);
					}
				break;
#endif
			case 0xFD:		// keep alive
					WIRE_putc(ERRORCOMMAND);
				break;
			case 0xFE:		// set buffer
					if (cmd_in[1] != 0x00)
						WIRE_putc(ERRORCOMMAND);
					else{
						/* get len of data block */
						fill_len = (cmd_in[2]<<8) + cmd_in[3];
						crc = 0;
						pBuf = dbuf;
						/* get data block with crc update */
						crc_getdata(pBuf, fill_len+2);
						if (crc == 0)
							WIRE_putc(SUCCESS);
						else
							WIRE_putc(ERRORCRC);
					}
				break;
			case 0xFF:		// set address
					destadr = (cmd_in[1]<<16) + (cmd_in[2]<<8) + cmd_in[3];
					WIRE_putc(SUCCESS);
				break;
			default:
					WIRE_putc(ERRORCOMMAND);
				break;
			}
		}while(1);
    }
	/* now start application firmware */
	StartApp();
}

// -------------------------------- EOF ----------------------------------------
